/*========================================================
 Ising: by Eduardo Sanchez Velasco. 03/20/98
 A Java Applet to simuate a 2-D Ising system.
 It does a very primitive heat bath Monte-Carlo.
 This applet uses antiflikering tricks in method update(). 
 Like any applet with Runnable threads, it implements:
  init()
  start()
  run() -> repaint() -> update() -> myPaint()
  stop()
  
 Call the applet from a HTML file with the line:
 <applet code="ising.class" width=600 height=300></applet>
==========================================================*/

import java.util.*;
import java.awt.*;
import java.applet.*;

  
public class ising extends Applet implements Runnable
{
   Thread ThisApplet = null;
   final int SIZE = 60;          // SIZE x SIZE 2-D Ising
   final int Width  = SIZE*10;   // applet width=2xheight
   final int Height = SIZE*5;    // applet height
   final Color Bgd = new Color(0XEA,0XFF,0XFF);
   final Color Up   = Color.blue;
   final Color Down = Color.red;
   final int Delay = 300;        // Milliseconds of delay
   Font Fnt;
   String labelT[] = {"0.0","0.5","1.0","1.5","2.0","2.5",
                      "3.0","3.5","4.0","4.5","5.0"};
   String labelH[] = {"0.0","0.1","0.2","0.3","0.4","0.5",
                      "0.6","0.7","0.8","0.9","1.0"};                   
   
   int Spin[][] = new int[SIZE][SIZE]; // Lattice of spins
   final double Tmax=5;          // Max temperature
   double T=Tmax/2;              // Temperature
   final double Hmax=1;          // Max magnetic field
   double H=0;                   // Magnetic field
   int  M=0;                     // Percent Magnetization
   double Pup[] = new double[9]; // Prob of up
   FontMetrics  Fmetric;
   final int XT0 = SIZE*5+10;   // Top x of thermometer
   final int Y0 = 15;           // Top y of thermometer
   final int W0 = 10;           // Thermometer width
   final int L0 = SIZE*5-2*Y0;  // Length of thermometer
   int XH0;                     // Top x of H bar 
   int H5;                      // Size of '5'
   
   // Private variables used in update() to avoid flikering
   private Image     offScreenImage;
   private Graphics  offScreenGraphics;
      
   // ----------------- METHODS ----------------------------
    
   public void init()
   {  
     // Set background color 
     setBackground(Bgd);
     
     // Create and init the off-screen image 
     offScreenImage = createImage(Width, Height);
     offScreenGraphics = offScreenImage.getGraphics();
     
     // Clear the board (do not use g.clearRect!)
     offScreenGraphics.setColor(Bgd);
     offScreenGraphics.fillRect(0,0,Width,Height);
     
     // Set the font
     Fnt = new Font("Helvetica", Font.PLAIN, 14);
     offScreenGraphics.setFont(Fnt);
     Fmetric = getFontMetrics(Fnt);
     H5 = Fmetric.charWidth('5')/2+1;
     XH0 = XT0+W0+2*Fmetric.stringWidth(labelT[0])+20;     
     
     // Init probabilities
     RehashPup();             // Get new Pup table
      
     // Init lattice with random spin
     for(int n1=0; n1<SIZE; n1++)
     for(int n2=0; n2<SIZE; n2++)
     if(Math.random()<=0.5) Spin[n1][n2] = +1; // up
     else                   Spin[n1][n2] = -1; // down 
     
   } //--------- End of init() -------------- 
   
   public void start() 
   { 
     if(ThisApplet == null)
     {
        ThisApplet = new Thread(this);
        ThisApplet.start();
     }
      
   } //--------- End of start() ------------ 
    
   // In run() we wait update the lattice and repaint 
   public void run() 
   {  
      int nn1,nn2, sum, Mag;
      while (ThisApplet != null) 
      {   
         repaint();     
         Mag=0;
         for(int n1=0; n1<SIZE; n1++)
         for(int n2=0; n2<SIZE; n2++)
         {
            sum = 0;
            nn1 = n1+1; if(nn1==SIZE) nn1=0;
            nn2 = n2;
            sum += Spin[nn1][nn2];
            nn1 = n1-1; if(nn1==-1) nn1=SIZE-1;
            sum += Spin[nn1][nn2];
            nn1 = n1;
            nn2 = n2+1; if(nn2==SIZE) nn2=0;
            sum += Spin[nn1][nn2];
            nn2 = n2-1; if(nn2==-1) nn2=SIZE-1;
            sum += Spin[nn1][nn2];
         
            if(Math.random()<=Pup[sum+4])
            { 
               Spin[n1][n2] = +1;
               Mag++;
            }   
            else
            {
               Spin[n1][n2] = -1;
               Mag--;
            }  
          }
          
          M = Math.round( (float)(Mag*100.0/(SIZE*SIZE)) );
           
          // Wait for a while
          try {Thread.sleep(Delay);} 
          catch (InterruptedException e){}
           
      }
      
   } //--------- End of run() -------------- 

   // Override update() with tricks to avoid flikering 
   public final synchronized void update(Graphics g)
   {
      myPaint(offScreenGraphics);
      g.drawImage(offScreenImage, 0, 0, null);
              
   } //--------- End of update() --------------  

   public void stop() 
   { 
       if( ThisApplet != null )
       {
           ThisApplet.stop();
           ThisApplet = null;
       }
   
    } //--------- End of stop() -------------- 
   
   // myPaint() Repaints the off-screen canvas
   public void myPaint(Graphics g) 
   {  
      
      // Clear the board (do not use g.clearRect!)
      g.setColor(Bgd);
      g.fillRect(0,0,Width,Height);
      
      // Paint the spins 
      for(int n1=0; n1<SIZE; n1++)
      for(int n2=0; n2<SIZE; n2++)
      {
          if(Spin[n1][n2]==1) g.setColor(Up); 
          else                g.setColor(Down);
          
          g.fillRect(n1*5, n2*5, 5, 5);    
      }
      
      // Paint the thermometer
      int LT = (int)Math.round(T*L0/Tmax); // Lenght of Hg
      g.setColor(Color.red);
      g.fillRoundRect(XT0,Y0+L0-LT, W0,LT, W0,W0);
      g.setColor(Color.black);
      g.drawRoundRect(XT0,Y0, W0,L0, W0,W0);
      for(int n=0; n<=10; n++)
      {
         g.drawLine(XT0+W0+5, Y0+L0-(n*L0)/10,
                    XT0+W0+15,Y0+L0-(n*L0)/10);
         g.drawString(labelT[n],
                      XT0+W0+20,H5+Y0+L0-(n*L0)/10);           
      }
      
      // Paint the magnetic field bar
      int LH = (int)Math.round(H*L0/Hmax); // Lenght of bar
      g.setColor(Color.green);
      g.fillRect(XH0,Y0+L0-LH, W0,LH);
      g.setColor(Color.black);
      g.drawRect(XH0,Y0, W0,L0);
      g.drawRect(XH0,Y0+L0+2,W0,3); // Extra box for zero
      for(int n=0; n<=10; n++)
      {
         g.drawLine(XH0+W0+5, Y0+L0-(n*L0)/10,
                    XH0+W0+15,Y0+L0-(n*L0)/10);
         g.drawString(labelH[n],
                      XH0+W0+20,H5+Y0+L0-(n*L0)/10);           
      }

      // Write info text
      int Xtext0 = 2*XH0-XT0+10;
      int Dy = 4*H5;
      String Value;
      Value = String.valueOf(T);
      if( Value.length() > 4 ) 
           Value = "T = " + Value.substring(0,4);
      else Value = "T = " + Value;      
      g.drawString(Value, Xtext0,Y0+H5);
      Value = String.valueOf(H);
      if( Value.length() > 5 ) 
           Value = "H = " + Value.substring(0,5);
      else Value = "H = " + Value;      
      g.drawString(Value, Xtext0,Y0+H5+Dy);
      if(M<=0)Value = "Mag = " + String.valueOf(M) +"%";
      else    Value = "Mag = +" + String.valueOf(M) +"%";      
      g.drawString(Value, Xtext0,Y0+H5+2*Dy);
         
   } //--------- End of Mypaint() --------------
   
   // Change value of T or H if mouse goes up in 
   // thermometer or H bar.
   public boolean mouseUp(Event click, int x, int y) 
   {
      // Check if (x,y) is inside thermometer
      if( x>=XT0 && x<=(XT0+W0) && y>=Y0 && y<=(Y0+L0) )
      {
         T = Tmax*(Y0+L0-y)/L0;   // New temperature
         RehashPup();             // Get new Pup table
         repaint();               // Update screen
         return true;
      }   
       
      // Check if (x,y) is inside H bar
      if( x>=XH0 && x<=(XH0+W0) && y>=Y0 && y<=(Y0+L0+6) )
      {
         H = Hmax*(Y0+L0-y)/L0;   // New temperature
         RehashPup();             // Get new Pup table
         repaint();               // Update screen
         return true;
      }   

        
      return true;
         
   }//--------- End of mouseUp() --------------
   
   // Update the probability table for spin up
   void RehashPup()
   {   
     if(T<=0) T = 0.001;   // Minimum T possible 
     if(H<=0.005) H = 0;         
     for(int n=0; n<9; n+=2)
        Pup[n] = Math.exp((n-4+H)/T)/
                 (Math.exp((n-4+H)/T)+Math.exp(-(n-4+H)/T));
   }           
}